define([
    "dojo/_base/declare",
    "thornberry/app/Component/_Component",
    "thornberry/app/Component/_EditorViewMixin",
    "dojo/_base/lang",
    "thornberry/app/Service/NoeNotr",
    "thornberry/app/Service/Patient",
    "thornberry/serverRequest",
    "thornberry/app/utils",
    "./StatusEditor"
],function(
    declare,
    _Component,
    _EditorViewMixin,
    lang,
    service,
    patientService,
    serverRequest,
    utils,
    StatusEditor
) {
    return declare([_Component,_EditorViewMixin], {

        title: "OASIS Acceptance Status",

        // _EditorContainerMixin props
        EditorClass: StatusEditor,

        // internals
        objectId: "",
        value: null,

        getInitialProps: function() {
            return lang.mixin(this.inherited(arguments),{
                type: "", // "NOE" or "NOTR",
                onSave: null // a method that this view uses to notify the parent when data has been saved
            });
        },

        getPageProps: function() {
            var t = this;
            return lang.mixin(t.inherited(arguments),{
                ndocNum: t.ndocNum,
                accountSummaryShown: true
            });
        },

        refreshRendering: function() {
            var t = this;
            if (t.isShown) {
                t.set("objectId",t.props.objectId);
                t.set("ndocNum",t.getNDocNum(t.props.objectId));

                t.editor.set("props",{
                    mode: "edit"
                });
            }
            t.inherited(arguments);
        },

        _setObjectIdAttr: function(objectId) {
            var t = this;
            if (objectId && t.objectId !== objectId) {
                t.objectId = objectId;

                t.addReadyPromise(serverRequest("Interface.ASAP.Acceptance","GetAssessmentStatus",objectId).then(function(value) {
                    if (t.objectId === objectId) {
                        t.value = value;
                        t.editor.set("props",{
                            value: value
                        });
                    }
                },utils.showMessage));
            }
            t.objectId = objectId;
        },

        _setNdocNumAttr: function(value) {
            var t = this;
            if (t.ndocNum != value) {
                t.ndocNum = value;
                t.addReadyPromise(patientService.getHeaderInfo(value));
            }
        },

        save: function() {
            var t = this;
            var data = lang.mixin(t.editor.get("value"),{
                documentId: t.value.documentId,
                ndocNum: t.value.ndocNum
            });
            return serverRequest("Interface.ASAP.Acceptance","SaveAssessmentStatus",data).then(function() {
                if (t.props.onSave) {
                    t.props.onSave();
                }
                t.objectId = null; // Force the editor to reload the object if the same object is edited immediately after
                t.value = null;
            });
        },

        getNDocNum: function(objectId) {
            return objectId ? objectId.toString().split("||")[0] : "";
        }
    });
});