define([
    "dojo/_base/declare",
    "thornberry/app/Component/_EditorComponent",
    "dijit/_TemplatedMixin",
    "dijit/_WidgetsInTemplateMixin",
    "dojo/_base/lang",
    "dojo/_base/array",
    "dojo/text!./Templates/processResponseEditor.html",
    "thornberry/app/utils",
    "./RecordGrid",
    "dojo/store/Memory",
    "dojo/aspect",
    "dojo/dom-style",
    "./RecordDetail",
    "dijit/form/DateTextBox",
    "dijit/form/Form",
    "dijit/form/Select"
],function(
    declare,
    _EditorComponent,
    _TemplatedMixin,
    _WidgetsInTemplateMixin,
    lang,
    arrayUtils,
    template,
    utils,
    RecordGrid,
    Memory,
    aspect,
    domStyle,
    RecordDetail
) {
    return declare([_EditorComponent,_TemplatedMixin,_WidgetsInTemplateMixin], {
        templateString: template,

        recordGrid: null,
        recordStore: null,
        recordDetails: null,

        constructor: function() {
            var t = this;
            t.recordStore = new Memory();
            t.recordDetails = [];
        },

        buildRendering: function() {
            var t = this;
            t.inherited(arguments);

            var grid = new RecordGrid({
                store: t.recordStore,
                selectableRows: true,
                onRowDoubleClick: function(id) {
                    //t.showDetail(id);
                }
            });
            t.recordGridContainer.appendChild(grid.domNode);
            grid.startup();
            //grid.connect(grid.select.row,"onSelectionChange",function() {t.onChangeSelection(grid.getSelected());});
            t.recordGrid = grid;

            // requirement logic
            t.addRequirement(null,true); // All fields required
        },

        postCreate: function() {
            var t = this;
            t.inherited(arguments);

            t.own(aspect.after(t.recordStore,"setData",lang.hitch(t,"updateRecordGrid")));
        },

        _setValueAttr: function(value) {
            var t = this;
            if (value !== t.value) {
                arrayUtils.forEach(t.recordGrid.select.row._getUnselectableRows(),function(id) {
                    t.recordGrid.select.row.setSelectable(id,true);
                });

                var records = (value && value.records ? value.records : []);
                records = arrayUtils.map(records,function(item,index) {
                    if (!item.NDocNum) {
                        t.recordGrid.select.row.setSelectable(index,false); // Don't allow processing of any rows that we couldn't match
                    }

                    item.id = index;
                    return item;
                });

                t.recordStore.setData(records);
                t.showDetails(records);

                t.getField("acceptanceDate").set("constraints",{
                    max: new Date(),
                    min: (t.props.value && t.props.value.minAcceptanceDate)
                });
            };
            t.inherited(arguments);
        },

        updateRecordGrid: function() {
            var t = this;
            t.recordGrid.select.row.clear();
            t.recordGrid.model.clearCache();
            t.recordGrid.body.refresh();
        },

        resize: function() {
            var t = this;
            t.recordGrid.resize();
        },

        showDetails: function(records) {
            var t = this;
            arrayUtils.forEach(records,function(record,index) {
                var component = t.recordDetails[index];
                if (!component) {
                    component = new RecordDetail();
                    t.recordDetailContainer.appendChild(component.domNode);
                    t.recordDetails[index] = component;
                    component.startup();
                } else {
                    domStyle.set(component.domNode,"display","");
                }
                component.set("props",{ value: record });
            });
            for (var i = records.length; i < t.recordDetails.length; i++) {
                // Hide any pre-existing record details beyond the ones just updated
                var component = t.recordDetails[i];
                domStyle.set(component.domNode,"display","none");
            }
        }
    });
});