define([
    "dojo/_base/declare",
    "thornberry/app/Component/_Component",
    "thornberry/app/Component/_EditorViewMixin",
    "dijit/_TemplatedMixin",
    "dijit/_WidgetsInTemplateMixin",
    "thornberry/app/Component/_DataFormMixin",
    "dojo/text!./Templates/ProcessResponseView.html",
    "dojo/_base/lang",
    "dojo/dom-style",
    "thornberry/app/utils",
    "dojo/_base/array",
    "dojo/date/locale",
    "./ProcessResponseEditor",
    "dojo/Deferred",
    "thornberry/serverRequest",
    "dijit/form/CheckBox"
],function(
    declare,
    _Component,
    _EditorViewMixin,
    _TemplatedMixin,
    _WidgetsInTemplateMixin,
    _DataFormMixin,
    template,
    lang,
    domStyle,
    utils,
    arrayUtils,
    locale,
    ProcessResponseEditor,
    Deferred,
    serverRequest
) {
    return declare([_Component,_EditorViewMixin,_TemplatedMixin,_DataFormMixin], {
        templateString: template,

        responseData: null,

        getInitialProps: function() {
            return lang.mixin(this.inherited(arguments),{
                responseData: []
            });
        },

        getPageProps: function() {
            var t = this;
            return lang.mixin(t.inherited(arguments),{
                content: t.domNode,
                actions : [
                    { label: "Cancel", onClick: t.props.onCloseView },
                    { label: "Import", onClick: lang.hitch(t,"process") }
                ],
                accountSummaryShown: false
            });
        },

        buildRendering: function() {
            var t = this;
            t.inherited(arguments);

            t.editor = new ProcessResponseEditor();
            t.editorContainer.appendChild(t.editor.domNode);
            t.addSubComponent(t.editor);
        },
 
        refreshRendering: function() {
            var t = this;
            t.title = lang.replace("Review OASIS Acceptance Response(s)",t.props);
            t.set("responseData",t.props.responseData);
            t.inherited(arguments);
        },

        process: function() {
            this.startSave().then(this.props.onCloseView);
        },

        save: function() {
            var t = this;
            var deferred = new Deferred();
            var selectedIds = t.editor.recordGrid.getSelected();
            if (selectedIds.length === 0) {
                deferred.reject("No responses selected to process.");
            } else {
                var responses = [];
                for (var i = 0; i < selectedIds.length; i++) {
                    var item = t.responseData.data[i];
                    responses.push({
                        ndocNum: item.NDocNum,
                        documentId: item.DocumentId,
                        acceptanceStatus: item.Status
                    });
                }

                var data = {
                    acceptanceDate: t.editor.get("value").acceptanceDate,
                    responses: responses
                };

                serverRequest("Interface.ASAP.CSVImport","Process",data).then(function() {
                    deferred.resolve();
                    if (t.props.onProcess) {
                        t.props.onProcess();
                    }
                },deferred.reject);
            }

            return deferred.promise;
        },

        _setResponseDataAttr: function(responseData) {
            var t = this;
            if (t.responseData !== responseData) {
                t.responseData = responseData;
                var value = responseData && responseData.data ? responseData.data[0] : null;
                t.set("value",value);

                var acceptanceDate = null;
                if (value) {
                    acceptanceDate = locale.format(new Date(value['"Completion Date/Time"']),{ selector: "date", datePattern: "yyyy-MM-dd" });
                }

                var minAcceptanceDate = (responseData.lastM0090 ? locale.parse(responseData.lastM0090,{ selector: "date", datePattern: "yyyy-MM-dd" }) : null);

                t.editor.set("props",{
                    value: {
                        acceptanceDate: acceptanceDate,
                        minAcceptanceDate: minAcceptanceDate,
                        records: (responseData ? responseData.data : [])
                    }
                });
            }
        },

        onShow: function() {
            this.editor.resize();
        },

        onWindowResize: function() {
            this.editor.resize();
        }
    });
});