define([
    "dojo/_base/declare",
    "dojo/_base/lang",
    "dojo/_base/array",
    "dojo/dom-style",
    "thornberry/app/component/Report/View",
    "thornberry/app/utils",
    "thornberry/app/Service/Settings",
    "thornberry/app/Service/NoeNotr",
    "thornberry/app/Service/EnsembleFactory"
],function(
    declare,
    lang,
    arrayUtils,
    domStyle,
    ReportView,
    utils,
    settingsService,
    service,
    ensembleFactory
) {
    return declare(ReportView,{

        settings: null,

        getTitle: function() {
            var t = this;
            return t.controller.report.displayName;
        },

        getInitialProps: function() {
            return lang.mixin(this.inherited(arguments),{
                reportClass: "Report.Definition.AsapAcceptance"
            });
        },

        getPageProps: function() {
            var t = this;
            var selectedIds = t.props.selectedIds;

            return lang.mixin(t.inherited(arguments),{
                actions: [
                    { label: "Refresh",             onClick: lang.hitch(t,"refreshData") },
                    { label: "Upload Response",     onClick: function() { t.loadSubView("uploadFileView"); } },

                    [
                        { label: "Edit",            onClick: lang.hitch(t,"_edit"),     hidden: selectedIds.length !== 1 }
                    ]
                ]
            });
        },

        refreshRendering: function() {
            var t = this;

            /*if (!t.settings) {
                t.addReadyPromise(settingsService.get("noeNotrTracking").then(function(settings) {
                    t.settings = settings;
                }));
            }*/

            t.inherited(arguments);
        },

        _edit: function() {
            this.edit(this.props.selectedIds[0]);
        },

        process: function() {
            var t = this;
            t.loadSubView("processResponse",{
                policyIds: t.props.selectedIds
            });
        },

        onRowDoubleClick: function(id) {
            this.edit(id); // Prevent if not allowed
        },

        edit: function(id) {
            var t = this;
            t.loadSubView("editView",{
                objectId: id
            });
        },

        onShow: function() {
            var t = this;
            t.inherited(arguments);

            if (t.firstShow) {
                t.addSubView("editView","thornberry/app/Component/AsapAcceptance/EditView",function() {
                    return { 
                        onSave: lang.hitch(t,"refreshData")
                    };
                },true);
                t.addSubView("uploadFileView","thornberry/app/Component/FileUploadView",function() {
                    return { 
                        title: "Upload OASIS Acceptance File",
                        serverClass: "Interface.ASAP.CSVImport",
                        serverMethod: "HandleFile",
                        onUpload: function(promise) {
                            t.props.onSetLoadingPromise(promise,"Processing...");
                            promise.always(t.getSubView("uploadFileView").props.onCloseView);
                            promise.then(function(responseData) {
                                if (!responseData || responseData.total == 0) {
                                    utils.showMessage("No results found.");
                                    return;
                                }

                                t.loadSubView("processResponse",{
                                    responseData: responseData
                                });
                            },utils.showMessage);   
                        }
                    };
                });
                t.addSubView("processResponse","thornberry/app/Component/AsapAcceptance/ProcessResponseView",function() {
                    return {
                        onProcess: function() {
                            t.refreshData();
                        }
                    };
                },true);
            }
        }
    });
});