define([
    "dojo/_base/declare",
    "dojo/_base/lang",
    "dojo/_base/array",
    "dijit/_WidgetBase",
    "thornberry/app/Component/_Component",
    "thornberry/app/Component/_ViewMixin",
    "thornberry/app/Component/_EditorViewMixin",
    "thornberry/app/Component/_EditorComponent",
    "thornberry/app/Component/_TableFieldEditorMixin",
    "dijit/_TemplatedMixin",
    "dijit/_WidgetsInTemplateMixin",
    "dojo/text!./Templates/CSVColumnSelectEditor.html",
    "dojo/text!./Templates/CSVImportResults.html",
    "dojo/text!./Templates/FieldSelector.html",
    "thornberry/app/Component/FileUploadView",
    "thornberry/app/utils",
    "gridx/Grid",
    "gridx/core/model/cache/Sync",
    "dojo/store/Memory",
    "dojo/dom-construct",
    "thornberry/serverRequest",
    "thornberry/app/utils",
    "dijit/form/Select"
],function(
    declare,
    lang,
    arrayUtils,
    _WidgetBase,
    _Component,
    _ViewMixin,
    _EditorViewMixin,
    _EditorComponent,
    _TableFieldEditorMixin,
    _TemplatedMixin,
    _WidgetsInTemplateMixin,
    template,
    importResultsTemplate,
    fieldSelectorTemplate,
    FileUploadView,
    utils,
    Grid,
    Cache,
    Memory,
    domConstruct,
    serverRequest,
    utils
) {
    var FieldSelector = declare([_WidgetBase,_TemplatedMixin,_WidgetsInTemplateMixin],{
        templateString: fieldSelectorTemplate
    });

    var ColumnSelectionEditor = declare([_EditorComponent,_TemplatedMixin,_WidgetsInTemplateMixin,_TableFieldEditorMixin],{
        templateString: template,

        grid: null,

        buildRendering: function() {
            var t = this;
            t.inherited(arguments);

            var data = t.props.availableColumns;
            data.unshift({ id: "[DNI]", name: "(DO NOT IMPORT)"});

            t.setTable("fields",data);

            t.totalNode.innerHTML = t.props.total;

            if (t.props.columns) {
                var structure = arrayUtils.map(t.props.columns,function(column,i) {
                    return {
                        id: i + 1,
                        field: column,
                        name: column
                    }
                });

                var i = 0;
                var store = new Memory({data:arrayUtils.map(t.props.data.slice(0,5),function(item) {item.id = i++; return item;})});

                t.grid = new Grid({
                    cacheClass: Cache,
                    store: store,
                    structure: structure,
                    autoHeight: true
                });
                t.gridContainer.appendChild(t.grid.domNode);

                arrayUtils.forEach(t.props.columns,function(column,i) {
                    var fieldSelector = new FieldSelector({
                        number: i+1,
                        label: column,
                        name: column
                    });
                    t.fieldContainer.appendChild(fieldSelector.domNode);
                });

                t.addTableFilter("",function(name,item) {
                    return arrayUtils.every(t.fields,function(field) {
                        return field.name === name || !field.tableName || item.id == "[DNI]" || field.get("value") != item.id;
                    });
                });

                t.addLogic("",function(name) {
                    arrayUtils.forEach(t.fields,function(field) {
                        if (field.name !== name && field.tableName) {
                            t.refreshTableField(field.name);
                        }
                    });
                });

                t.addRequirement("",true);
                t.addDisabled("fileName",true);
            }
        },

        startup: function() {
            this.grid.startup();
        },

        refreshRendering: function() {
            var t = this;
            t.inherited(arguments);

            t.set("value",{
                fileName: t.props.fileName
            });
        },

        resize: function() {
            this.grid.resize();
        }
    });

    var ColumnSelectionView = declare([_Component,_ViewMixin],{
        getTitle: function() {
            return this.props.title;
        },

        getInitialProps: function() {
            var t = this;
            return lang.mixin(t.inherited(arguments),{
                fileName: "",
                columns: [],
                data: [],
                availableColumns: []
            });
        },

        getPageProps: function() {
            var t = this;
            return lang.mixin(t.inherited(arguments),{
                content: t.domNode,
                actions: [
                    { label: "Back", onClick: t.props.goBack },
                    [
                        { label: "Import", onClick: lang.hitch(t,"importData") }
                    ]
                ]
            });
        },

        buildRendering: function() {
            var t = this;
            t.inherited(arguments);
            t.domNode = domConstruct.create("div");
        },

        refreshRendering: function() {
            var t = this;
            if (t.editor) {
                t.editor.destroy();
            }
            t.editor = new ColumnSelectionEditor({
                props: t.props
            });
            t.domNode.appendChild(t.editor.domNode);
            t.editor.startup();
        },

        importData: function() {
            var t = this;
            var mapping = t.editor.get("value");
            delete mapping.fileName;
            if (!t.editor.checkRequired()) {
                utils.showMessage("Please fill in all required fields.");
            } else if (t.checkMapping(mapping)) {
                var data = {
                    data: t.props.data,
                    mapping: mapping
                }
                t.props.onImport(data);
            }
        },

        checkMapping: function(data) {
            var t = this;
            var valueIndex = {};
            for(var name in data) {
                valueIndex[data[name]] = name;
            }
            
            var missing = [];
            arrayUtils.forEach(t.props.availableColumns,function(column) {
                if (column.required && !(column.id in valueIndex)) {
                    missing.push(column.name);
                }
            });
            if (missing.length) {
                utils.showMessage("The following fields must be mapped to a source column: "+missing.join(", "),"");
            }
            return !missing.length;
        },

        resize: function() {
            this.editor.resize();
        }
    });

    var Error = declare([_WidgetBase,_TemplatedMixin],{
        templateString: 
            "<div class='content-flex-container'>"+
                '<div class="flex-field-container vertical-field-align">'+
                    '<div class="field-name field-name-medium">'+
                        '<label>${label}</label>'+
                    '</div>'+
                    '<div class="field-value" style="padding:5px;padding-left:25px;">'+
                        '<label>${message}</label>'+
                    '</div>'+
                '</div>'+
            "</div>"
    });

    var ImportResultView = declare([_Component,_ViewMixin,_TemplatedMixin],{
        templateString: importResultsTemplate,

        getTitle: function() {
            return this.props.title;
        },

        getInitialProps: function() {
            var t = this;
            return lang.mixin(t.inherited(arguments),{
                results: {
                    errors: [],
                    summary: {
                        records: 0,
                        imported: 0,
                        errors: 0
                    }
                }
            });
        },

        getPageProps: function() {
            var t = this;
            return lang.mixin(t.inherited(arguments),{
                content: t.domNode,
                actions: [
                    { label: "Done", onClick: t.props.onCloseView },
                ]
            });
        },

        refreshRendering: function() {
            var t = this;
            t.recordsNode.innerHTML = t.props.results.summary.records;
            t.importedNode.innerHTML = t.props.results.summary.imported;
            t.errorsNode.innerHTML = t.props.results.summary.errors;

            console.log(t.props.results.errors);
            domConstruct.empty(t.errorContainer);
            arrayUtils.forEach(t.props.results.errors,function(error) {
                var errorItem = new Error(error);
                t.errorContainer.appendChild(errorItem.domNode);
            });
        }
    });

    return declare([_Component,_ViewMixin],{
        title: "",
        currentView: null,
        cacheClass: "",

        fileUploadView: null,
        fileName: "",

        getTitle: function() {
            return this.currentView && this.currentView.getTitle();
        },

        getInitialProps: function() {
            return lang.mixin(this.inherited(arguments),{
                cacheClass: "",         // A class that extends Utility.CSVImport
                onComplete: null        // A function to be called when the import is complete
            });
        },

        getPageProps: function() {
            var t = this;
            return lang.mixin(t.inherited(arguments),t.currentView.getPageProps());
        },

        refreshRendering: function() {
            var t = this;
            t.inherited(arguments);
            t.set("cacheClass",t.props.cacheClass);
        },

        onShow: function() {
            var t = this;
            if (t.currentView) {
                t.currentView.onShow();
            }
        },

        onHide: function() {
            var t = this;
            if (t.fileUploadView) {
                t.set("currentView",t.fileUploadView);
            }
        },

        resize: function() {
            var t = this;
            if (t.currentView) {
                t.currentView.resize();
            }
        },

        _setCacheClassAttr: function(value) {
            var t = this;
            if (value && t.fileUploadView === null) {
                t.fileUploadView = new FileUploadView({
                    onSelectFile: function(filename) {
                        var t = this;
                        var filenamePieces = filename.toString().split(".");
                        if (filenamePieces.length < 2 || filenamePieces[filenamePieces.length - 1].toUpperCase() !== "CSV") {
                            utils.showMessage("Please upload a .CSV file")
                        } else {
                            t.inherited("onSelectFile",arguments);
                        }
                    },
                    props: {
                        title: lang.replace("Import {title}",t.props),
                        serverClass: value,
                        serverMethod: "HandleFile",
                        onUpload: lang.hitch(t,"onUpload"),
                        onUpdatePageProps: t.props.onUpdatePageProps,
                        onCloseView: t.props.onCloseView,
                        goBack: lang.hitch(t,"goBack")
                    }
                });
                t.set("currentView",t.fileUploadView);
            }
        },

        _setCurrentViewAttr: function(value) {
            var t = this;
            if (t.currentView !== value && value) {
                if (t.currentView) {
                    t.currentView.onHide();
                }
                value.onBeforeShow();
                t.currentView = value;
                t.setPageProps();
            }
        },

        onUpload: function(uploadPromise) {
            var t = this;
            t.props.onSetLoadingPromise(uploadPromise);
            uploadPromise.then(function(result) {
                if (result.total == 0) {
                    utils.showMessage("This file contains 0 records.");
                } else {
                    t.fileName = result.fileName;
                    if (!t.columnSelectionView) {
                        t.columnSelectionView = new ColumnSelectionView({
                            props: {
                                onUpdatePageProps: t.props.onUpdatePageProps,
                                goBack: lang.hitch(t,"goBack"),
                                onImport: lang.hitch(t,"onImport")
                            }
                        });
                    }
                    result.title = lang.replace("Import {title} - {fileName}",{ title: t.props.title, fileName: t.fileName }),
                    t.columnSelectionView.set("props",result);
                    t.set("currentView",t.columnSelectionView);
                }
            },utils.showMessage);
        },

        onImport: function(data) {
            var t = this;
            var promise = serverRequest(t.props.cacheClass,"ImportData",data).then(function(results) {
                if (!t.importResultView) {
                    t.importResultView = new ImportResultView({
                        props: {
                            title: lang.replace("Import Summary - {fileName}",t),
                            results: results,
                            onUpdatePageProps: t.props.onUpdatePageProps,
                            onCloseView: function() {
                                t.props.onCloseView();
                                if (t.props.onComplete) {
                                    t.props.onComplete();
                                }
                            }
                        }
                    });
                } else {
                    t.importResultView.set("props",{ results: results, title: lang.replace("Import Summary - {fileName}",t)});
                }
                t.set("currentView",t.importResultView);
            },utils.showMessage);
            t.props.onSetLoadingPromise(promise);
        },  

        goBack: function() {
            var t = this;
            if (t.currentView === t.fileUploadView) {
                t.props.onCloseView();
            } else if (t.currentView === t.columnSelectionView) {
                t.set("currentView",t.fileUploadView);
            }
        }
    });
});