if (typeof ndoc === 'undefined') {
	ndoc = {};
	ndoc.window = {};
	ndoc.window.resizeTo = function(width, height, win) {
	 /*
		Purpose: Resize the window
		Inputs: width - As %Integer [in px]
				height - As %Integer [in px]
		Outputs: none
		Assumptions: none
		Notes: Microsoft KB904947 - resizeTo() will throw an 'Access Denied' error if called while mouse button is down
									The try..catch implemented here will asynchronously call the resizeTo function
	 */
		var offsetWidth, offsetHeight;
		var win = win || window;
		try {
			if (win.resizeTo) {
				if ((width != '100%') || (height != '100%')) {
					// Resize viewport rather than window
					if (win.outerWidth) {
						offsetWidth = (win.outerWidth - win.innerWidth);
						offsetHeight = (win.outerHeight - win.innerHeight);
					} else {
						offsetWidth = 20;
						offsetHeight = 65;
					}
				}
				if (width == '100%') { width = win.screen.availWidth; offsetWidth = 0; }
				if (height == '100%') { height = win.screen.availHeight; offsetHeight = 0; }
				win.resizeTo(width + offsetWidth, height + offsetHeight);
			}
		} catch(e) {
			window.setTimeout(function(){ndoc.window.resizeTo(width, height, win)}, 1);
		}
		return;
	};
	ndoc.window.moveTo = function(x, y, win) {
	 /*
		Purpose: Resize the window
		Inputs: x - As %Integer [in px] - relative to left
				y - As %Integer [in px] - relative to top
		Outputs: none
		Assumptions: none
		Notes: Microsoft KB904947 - moveTo() will throw an 'Access Denied' error if called while mouse button is down
									The try..catch implemented here will asynchronously call the moveTo function
	 */
		var win = win || window;
		try {
			if (win.moveTo) {
				win.moveTo(x, y);
			}
		} catch(e) {
			window.setTimeout(function(){ndoc.window.moveTo(x, y, win)}, 1);
		}
		return;
	};
}

if (typeof checkMaxLength === 'undefined') {
	if (typeof MAXLENGTH === 'undefined') { var MAXLENGTH = 15000; }
	var MAXLENGTH_WARNING = 0;
	var checkMaxLength = function(name) {
		var element = document.getElementsByName(name)[0];
		var truncateCtr = 0
		var textLength = element.value.length;
		var text = element.value;
		
		// Truncate string by 1 character and check the length of the escaped value each time to see if it is now less than MAXLENGTH.
		var truncatedLength = escape(text).length;
		while (truncatedLength > MAXLENGTH) {
			text = text.substr(0, (text.length - 1));
			truncateCtr++;
			truncatedLength = escape(text).length;
		}
		element.value = text;
		
		// Warn if this is the first time.
		if ((MAXLENGTH_WARNING === 0) && (truncateCtr > 0)) {
			var amountOver = textLength - text.length;
			alert('The text exceeds the maximum length of ' + MAXLENGTH.toString() + ' characters by ' + amountOver.toString() + ' characters. Text has been truncated.' );
			//alert('The text you have entered exceeds the maximum character limit of '+MAXLENGTH+' for storage of this field.  '+amountOver+' character(s) were removed to meet the text length restriction.');
			// Removed to allow continuous warnings
			//MAXLENGTH_WARNING = 1;
		}
		return;
	};
}

if (typeof Disable === 'undefined') {
	var Disable = function(TYP, DDE, TF) {
		require(["dijit/registry"], function(registry) {
			if ((TYP === 'S') || (TYP === 'T') || (TYP === 'A') || (TYP === 'B')) {
				// Update dijit textbox if it exists
				if (registry.byId(DDE + TYP)) {
					registry.byId(DDE + TYP).set('disabled', TF);
				} else {
					document.getElementsByName(DDE)[0].disabled = TF;
				}
			} else if ((TYP === 'R') || (TYP === 'C')) {
				var elements = document.getElementsByName(DDE);
				for (var i = 0; i < elements.length; i++) {
					elements[i].disabled = TF;
				}
			}
		});
	};
}
if (typeof Upd === 'undefined') {
	var RELOAD1 = '',
		PREFUNC = '',
		SOUND = '',
		CONFIRM = '',
		PROMPT = '',
		DSTR = 0,
		REQ = 0,
		NRQ = 0,
		HID = 0,
		HIDE=0,
		VIS = 0,
		SHOW = 0,
		DIS = 0,
		ENA = 0,
		LABELCOLOR = '',
		INNERHTML = '',
		LABELS = '',
		RELOAD = '',
		FOC = 0,
		CRV = '',
		POSFUNC = '',
		MSG = '',
		MSG1 = '';
	var Upd = function(){
	 /*
		Purpose: Update the page after a server call (e.g. field validation causes other fields to become required)
		Inputs: A large set of variables in the global scope
		Outputs: none
		Assumptions: Variable dependencies (i.e. some variables are expected to be set if another has a particular value)
					Javascript variable reloadUrl exists on pages using this Upd() function
		Notes: 
	 */
		require(["dijit/registry"], function(registry) {
			var TYP = '';
			var LGTH;
			var RDE;
			var DDE;
			var ELE;
			// Check if we have to reload first
			if (RELOAD1 === '1') {
				window.location = reloadUrl;
				RELOAD1 = '';
			} else {
				// Should I run a function before updating the rest?
				if (PREFUNC !== '') {
					eval(PREFUNC);
					PREFUNC = '';
				}
				// Confirm an action
				if (CONFIRM !== '') {
					var response = confirm(CONFIRM);
					confirmValue(response);
					CONFIRM = '';
				}
				// Prompt for more detail
				if (PROMPT !== '') {
					var response = prompt(PROMPT);
					promptValue(response);
					PROMPT = '';
				}
				// Select options from a select box, checkbox, or radio
				if (DSTR !== 0) {
					var DARRAY = DSTR.split('|');
					var OARRAY = OSTR.split('|');
					for (var k = 0; k < DARRAY.length; k++) {
						var D = DARRAY[k];
						var O = OARRAY[k];
						TYP = D.slice(0, 1);
						D = D.slice(1, 99);
						if (TYP === 'S') {
							var selectBox = document.getElementsByName(D)[0];
							selectBox.selectedIndex = -1;
							var sel = O.split('~');
							if (sel !== '') {
								for (var i = 0; i < sel.length; i++) {
									for (var j = 0; j < selectBox.options.length; j++) {
										if (selectBox.options[j].value === sel[i]) {
											selectBox.options[j].selected = true;
										}
									}
								}
							}
						} else if (TYP === 'R') {
							if (O !== '') {
								document.getElementById(D + "R" + O).checked = 1;
							} else {
								var elements = document.getElementsByName(D);
								for (var j = 0; j < elements.length; j++) {
									var element = elements[j];
									element.style.checked = 0;
								}
							}
						} else if (TYP === 'C') {
							var elements = document.getElementsByName(D);
							for (var j = 0; j < elements.length; j++) {
								var element = elements[j];
								element.style.checked = 0;
							}
							
							var sel = O.split('~');
							for (var i = 0; i < sel.length; i++) {
								opt = sel[i];
								if (opt !== '') {
									document.getElementById(D + 'C' + opt).checked = 1;
								}
							}
						}
					}
					DSTR = 0;
					OSTR = 0;
				}
				// Set fields to be required
				if (REQ !== 0) {
					var LIST = REQ.split('|');
					for (var i = 0; i < LIST.length; i++) {
						RDE = LIST[i];
						TYP = RDE.slice(0, 1);
						DDE = RDE.slice(1, 99);
						if ((TYP === 'R') || (TYP === 'C')) {
							var elements = document.getElementsByName(DDE);
							for (var j = 0; j < elements.length; j++) {
								var element = elements[j];
								element.style[RSTYL] = RVAL;
							}
						} else {
							document.all[DDE].style[RSTYL] = RVAL;
							// Update dijit textbox if it exists
							if (registry.byId(DDE + 'T')) {
								var style = 'background-color:' + RVAL;
								registry.byId(DDE + 'T').set('style', style);
							}
						}
					}
					REQ = 0;
				}
				// Set fields to no longer be required
				if (NRQ !== 0) {
					var LIST = NRQ.split('|');
					for (var i=0; i < LIST.length; i++) {
						RDE = LIST[i];
						TYP = RDE.slice(0, 1);
						DDE = RDE.slice(1, 99);
						if ((TYP === 'R') || (TYP === 'C')) {
							var elements = document.getElementsByName(DDE);
							for (var j = 0; j < elements.length; j++) {
								var element = elements[j];
								element.style.backgroundColor = "";
							}
						} else {
							document.all[DDE].style['backgroundColor'] = 'WHITE';
							// Update dijit textbox if it exists
							if (registry.byId(DDE + 'T')) {
								var style = 'background-color: white';
								registry.byId(DDE + 'T').set('style', style);
							}
						}
					}
					NRQ = 0;
				}
				// Hide fields
				if (HID !== 0) {
					var LIST = HID.split('|');
					for (var i = 0; i < LIST.length; i++) {
						RDE = LIST[i];
						TYP = RDE.slice(0, 1);
						DDE = RDE.slice(1, 99);
						if ((TYP === 'R') || (TYP === 'C')) {
							var elements = document.getElementsByName(DDE);
							for (var j = 0; j < elements.length; j++) {
								var element = elements[j];
								element.style.visibility = 'hidden';
							}
						} else {
							document.all[DDE].style.visibility = 'hidden';
						}
					}
					HID = 0;
				}
				if (HIDE !== 0) {
					var LIST = HIDE.split('|');
					for (var i = 0; i < LIST.length; i++) {
						RDE = LIST[i];
						TYP = RDE.slice(0, 1);
						DDE = RDE.slice(1, 99);
						if ((TYP === 'R') || (TYP === 'C')) {
							var elements = document.getElementsByName(DDE);
							for (var j = 0; j < elements.length; j++) {
								var element = elements[j];
								element.style.visibility = 'hidden';
							}
						} else {
							document.getElementsByName(RDE)[0].style.visibility = 'hidden';
						}
					}
					HIDE = 0;
				}
				// Show fields
				if (VIS !== 0) {
					var LIST = VIS.split('|');
					for (var i = 0; i < LIST.length; i++) {
						RDE = LIST[i];
						TYP = RDE.slice(0, 1);
						DDE = RDE.slice(1, 99);
						if ((TYP === 'R') || (TYP === 'C')) {
							var elements = document.getElementsByName(DDE);
							for (var j = 0; j < elements.length; j++) {
								var element = elements[j];
								element.style.visibility = 'visible';
							}
						} else {
							document.all[DDE].style.visibility = 'visible';
						}
					}
					VIS = 0;
				}
				if (SHOW !== 0) {
					var LIST = SHOW.split('|');
					for (var i = 0; i < LIST.length; i++) {
						RDE = LIST[i];
						TYP = RDE.slice(0, 1);
						DDE = RDE.slice(1, 99);
						if ((TYP === 'R') || (TYP === 'C')) {
							var elements = document.getElementsByName(DDE);
							for (var j = 0; j < elements.length; j++) {
								var element = elements[j];
								element.style.visibility = 'visible';
							}
						} else {
							document.getElementsByName(RDE)[0].style.visibility = 'visible';
						}
					}
					SHOW = 0;
				}
				// Disable fields
				if (DIS !== 0) {
					var LIST = DIS.split('|');
					for (var i = 0; i < LIST.length; i++) {
						ELE = LIST[i];
						TYP = ELE.slice(0, 1);
						DDE = ELE.slice(1, 99);
						Disable(TYP, DDE, 1);
					}
					DIS = 0;
				}
				// Enable fields
				if (ENA !== 0) {
					var LIST = ENA.split('|');
					for (var i = 0; i < LIST.length; i++) {
						ELE = LIST[i];
						TYP = ELE.slice(0, 1);
						DDE = ELE.slice(1, 99);
						Disable(TYP, DDE, 0);
					}
					ENA = 0;
				}
				// Change label text color
				if (LABELCOLOR !== '') {
					var LARRAY = LABELNAMES.split('|');
					var CARRAY = LABELCOLOR.split('|');
					for (var i = 0; i < LARRAY.length; i++) {
						ELE = LARRAY[i];
						COL = CARRAY[i];
						document.getElementsByName(ELE)[0].style.color = COL;
					}
					LABELCOLOR = '';
					LABELNAMES = '';
				}
				// Change innerHTML on elements
				if (INNERHTML !== '') {
					var LARRAY = INNERNAMES.split('|');
					var IARRAY = INNERHTML.split('|');
					for (var i = 0; i < LARRAY.length; i++) {
						ELE = LARRAY[i];
						HTML = IARRAY[i];
						document.getElementsByName(ELE)[0].innerHTML = HTML;
					}
					INNERHTML = '';
					INNERNAMES = '';
				}
				// Change label text
				if (LABELS !== '') {
					var LABLIST = LABELS.split('|');
					var LABVALS = LABELVALUES.split('|');
					for (var i = 0; i < LABLIST.length; i++) {
						document.getElementsByName(LABLIST[i])[0].innerHTML = LABVALS[i].replace('^', ' ');
					}
					LABELS = '';
					LABELVALUES = '';
				}
				// TODO: What's the practicality in this? Wouldn't you just reload first? Maybe for CONFIRM/PROMPT, but it could be moved up higher
				// Reload after updating
				if (RELOAD === '1') {
					window.location = reloadUrl;
					RELOAD = '';
				}
				// Give focus to a field
				if (FOC !== 0) {
					//str = 'document.getElementsByName(' + FOC + ')[0].focus()';
					document.getElementsByName(FOC)[0].focus();
					//setTimeout('eval(str)', 5);
					FOC = 0;
				}
				// Clear return value
				if (CRV !== '') {
					if (window.event) {
						window.event.returnValue = '';
					}
					CRV = '';
				}
				// Execute function after updates
				if (POSFUNC !== '') {
					eval(POSFUNC);
					POSFUNC = '';
				}
				// Show an alert
				if (MSG !== '') {
					alert(MSG);
					MSG = '';
				}
				// TODO: This would be really annoying (getting two alerts in a row) 
				// TODO:  -->RE: I concur with this notion of being annoying and such
				if (MSG1 !== '') {
					alert(MSG1);
					MSG1 = '';
				}
			}
		});
	}
}

function checkForT(dateId) {
	require(["dojo/dom", "dijit/registry"], function(dom, registry) {
		var dv = dom.byId(dateId).value.toString();
		if (dv.substring(0, 1).toLowerCase() === "t") {
			var date = new Date();
			var offset = dv.substring(1);
			// Check for T+# or T-#
			if (offset != "") {	
				// Convert substring to number
				offset = +offset;
				date.setDate(date.getDate() + offset);
			}
		} else if (dv.match(/^[0-9]{4}$/g)) { // mmdd
			var date = new Date(new Date().getFullYear(), (dv.substring(0,2) - 1), dv.substring(2,4));
		} else if (dv.match(/^[0-9]{6}$/g)) { // mmddyy
			var fullYear = new Date().getFullYear().toString();
			var centuryChange = 5 + (fullYear.substring(2,4) * 1); // change to previous century if more than 5 years past current year's last 2-digits
			var curCentury = dv.substring(4,6) * 1
			if (curCentury < centuryChange) {
				fullYear = fullYear.substring(0,2) + dv.substring(4,6);
			} else {
				fullYear = (fullYear.substring(0,2) - 1) + dv.substring(4,6);
			}
			var date = new Date(fullYear, (dv.substring(0,2)-1), dv.substring(2,4));
		} else if (dv.match(/^[0-9]{8}$/g)) { // mmddyyyy
			var date = new Date(dv.substring(4,8), (dv.substring(0,2)-1), dv.substring(2,4));
		} else if (dv.match(/^[0-9]{2}[/]{1}[0-9]{2}$/g)) { // mm/dd
			var date = new Date(new Date().getFullYear(), (dv.substring(0,2) - 1), dv.substring(3,5));
		} else if (dv.match(/^[0-9]{1}[/]{1}[0-9]{2}$/g)) { // m/dd
			var date = new Date(new Date().getFullYear(), (dv.substring(0,1) - 1), dv.substring(2,4));
		} else if (dv.match(/^[0-9]{2}[/]{1}[0-9]{1}$/g)) { // mm/d
			var date = new Date(new Date().getFullYear(), (dv.substring(0,2) - 1), dv.substring(3,4));
		} else if (dv.match(/^[0-9]{1}[/]{1}[0-9]{1}$/g)) { // m/d
			var date = new Date(new Date().getFullYear(), (dv.substring(0,1) - 1), dv.substring(2,3));
		}
		
		if (typeof date !== "undefined") {
			// update DateTextBox with new value
			registry.byId(dateId).set("value", date);
		}
	});
}

function checkForN(timeId) {
	require(["dojo/dom", "dijit/registry"], function(dom, registry) {
		var tv = dom.byId(timeId).value.toString();
		if (tv.substring(0, 1).toLowerCase() === "n") {
			var date = new Date();
			var offset = tv.substring(1);
			// Check for N+# or N-#
			if (offset != "") {	
				// Convert substring to number
				offset = +offset;
				date.setMinutes(date.getMinutes() + offset);
			}
		} else if (tv.match(/^([0-9]{1}|[0-9]{2})$/g)) { // mmdd
			var date = new Date();
			date.setHours(tv);
			date.setMinutes(0);
		} 
		
		if (typeof date !== "undefined") {
			// update DateTextBox with new value
			registry.byId(timeId).set("value", date);
		}
	});
}

// parse a date in yyyy-mm-dd format
function parseDate(input) {
	var parts = input.match(/(\d+)/g);
	// new Date(year, month [, date [, hours[, minutes[, seconds[, ms]]]]])
	return new Date(parts[0], parts[1]-1, parts[2]); // months are 0-based
}

// formatter for displaying date as mm/dd/yyyy
function dateFormatter(date) {
	if ((typeof date) === "undefined" || date === "N/A" || date === "" || date === "...") return date;
	var dateObj = parseDate(date);
	var dateString = dateFormat(dateObj, "MM/DD/YYYY");
	return dateString;
	
}

function paragraphFormatter(text) {
	var formattedValue = "";
	if (typeof text !== "undefined" && text !== null) {
		var textArr = text.toString().split("\n");
		
		for (var i = 0; i < textArr.length; i++) {
			if (i > 0) {
				formattedValue += "<br>";
			}
			formattedValue += escapeText(textArr[i]);
		}
	}
	return formattedValue;
}

function dateFormat(date, format) {
	// Calculate date parts and replace instances in format string accordingly
	format = format.replace("DD", (date.getDate() < 10 ? '0' : '') + date.getDate()); // Pad with '0' if needed
	format = format.replace("MM", (date.getMonth() < 9 ? '0' : '') + (date.getMonth() + 1)); // Months are zero-based
	format = format.replace("YYYY", date.getFullYear());
	return format;
}

function escapeText(input) {
	var output
	require(["dojox/html/entities"], function(entities) {
		output = entities.encode(input.toString());
	});
	return output;
}

// Purpose: used to gather all subscripts of JSON.
// NOTE: Whatever page uses this should have a global variable called "helperJson" defined. This function does not return a value because it is called asynchronously.
// finish: This parameter is a function object that is passed. It will be called upon completion of the function. It should make use of the helperJson variable and finish any other page loading that is necessary.
function gatherJSON(jsonTitle, finish, deferred) {
	// alternate approach - use serverRequest() if page uses dojo and the uploadForm node is present
	if (("undefined" !== typeof require) && (document.getElementById("uploadForm") !== null)) {
		require(["thornberry/serverRequest"], function(serverRequest) {
			serverRequest("UI.Utility","GatherJSON",jsonTitle,true).then(function(result) {
				finish(result.json,deferred);
			},console.log);
		});
		return;
	} 
	
	
	// initialize JSON string
	json = "";
	// loop Cache calls to get strings
	var i = 1;

	var selfRecurse = function() { gatherJSON(jsonTitle, finish, deferred) };

	// recursive calls waiting for JSON to finish loading
	var JSONloaded = checkJsonLoaded(jsonTitle);
	if (JSONloaded == 1) {
		_gatherJSON(jsonTitle, finish, 1, false, deferred);
	} else {
		setTimeout(selfRecurse, 1000);
	}
}

function _gatherJSON(jsonTitle, finish, i, json, deferred) {
	var selfRecurse = function() { _gatherJSON(jsonTitle, finish, i, json, deferred) };
	loaded = 0;
	jsonDone = 0;
	jsonPart = "";
	var loaded = loadJSONPart(jsonTitle, i)

	// recursive calls
	if (loaded == 1 && jsonPart != "" && jsonDone != 1) { // should run at least once
		if (i == 1) {
			json = "";
		}
		json += jsonPart;
		i++;
		setTimeout(selfRecurse);
	} else if (loaded != 1) {
		setTimeout(selfRecurse);
	} else if (jsonDone == 1) { // loaded == 1 && jsonPart == ""
		finish(json,deferred);
	}
	return;
}

function sendJSON(jsonTitle, json, maxSendSize) {
	var DATAMAXLEN = maxSendSize || 10000;
	json = json.replace(/\"/g, "\"\"");
	var start = 0;
	var last = 0;
	var success = 1;
	var first = 1
	while(!last && success) {
		var end = start + DATAMAXLEN;
		var jsonPiece = json.substring(start, end);
		
		// Count + characters in the string and limit to 255. Why? I don't know, but more than that causes problems
		var tempArray = jsonPiece.split("+");
		var endIndex = tempArray.length;
		if (endIndex > 255) {
			endIndex = 255;
		}
		var tempArray = tempArray.slice(0,endIndex);
		var jsonPiece = tempArray.join("+");
		end = jsonPiece.length;
		if ((jsonPiece.charAt(end-1) === "\"") && (jsonPiece.charAt(end-2) !== "\"")) {
			jsonPiece = jsonPiece.substring(0,jsonPiece.length-1);
			end -= 1;
		}
		
		end += start;
		start = end;
		if (end >= json.length) {
			last = 1;
		}
		success = sendJSONPart(jsonTitle, jsonPiece, first);
		first = 0;
	}
	return success
}

// Handles errors from Javascript
function errorDisplay(error) {
	txt = "There was an error on this page.\n\n";
	if (typeof(error) == "string") {
		txt += "Error description: " + error + "\n";
	} else {
		txt += "Error description: " + error.description + "\n";
	}
	txt += "Click OK to continue.\n\n";
	alert(txt);
	return;
}

 /* Purpose:  Handle sending text area values to cache server.  Checks the max length of the field passed and for invalid '~' characters.
	Assumptions:  Methods SetTextArea() and FinalizeTextArea() must be defined on the calling ASP page
							e.g. var SetTextArea = function(fld,str) { Cache:SETTXT(fld,str); };
								   var FinalizeTextArea = function(fld) { Cache:FINTXT(fld); };
							Note: Both methods can call the same function (e.g. Cache:SETTXT()) if it makes sense in the calling ASP page design.
 */
function SendTextAreaVal(fld) { 
	var msgText = document.getElementsByName(fld)[0].value;
	checkMaxLength(fld);
	
	// remove '~' characters
	if (msgText.match("~")) {
		alert("~ is an invalid character.") ;
		msgText = msgText.replace(/~/g, "");
		document.getElementsByName(fld)[0].value = msgText;
	}
	
	// send value in segments
	var SplitValue = function(fld, msgText) {
		var limit = 7500;
		if (msgText.length == 0) {
			FinalizeTextArea(fld); // Define on calling ASP page
			Upd();
			return;
		}
		var str = msgText.substring(0, limit);
		msgText = msgText.substring(limit, msgText.length);
		SetTextArea(fld, str);  // Define on calling ASP page
		SplitValue(fld, msgText);
	}
	SplitValue(fld, msgText);
	return;
}