define([
    "dojo/_base/declare",
    "thornberry/report/Controller",
    "thornberry/report/SummarizableController",
    "thornberry/report/CrossTabGridView",
    "thornberry/object/ObjectEditor",
    "thornberry/object/template",
    "dijit/layout/StackContainer",
    "dijit/layout/BorderContainer",
    "dijit/layout/ContentPane",
    "dijit/registry",
    "dojo/_base/array",
    "dojo/_base/lang",
    "dojo/dom-construct",
    "dojo/dom-style",
    "dojo/store/Memory",
    "dojo/on",
    "gridx/Grid",
    "gridx/core/model/cache/Sync",
    "gridx/modules/SingleSort",
    "gridx/modules/Bar",
    "gridx/modules/ColumnResizer",
    "dojo/aspect",
    "dijit/form/Select",
    "dojo/Stateful"
],function(declare,Controller,SummarizableController,CrossTabGridView,ObjectEditor,template,StackContainer,BorderContainer,ContentPane,registry,arrayUtils,lang,domConstruct,domStyle,Memory,on,Grid,Cache,SingleSort,Bar,ColumnResizer,aspect,Select,Stateful) {
	var CrossTabReportController = declare([Controller], {
        summarizeByModified: false,

        postMixInProperties: function() {
            var t = this;
            t.inherited(arguments);

            t.summaryStore = new Memory({
                idProperty:"Name"
            });

            t.modified.summarization = true;
        },

        buildRendering: function() {
            var t = this;
            t.inherited(arguments);

            t._createSummaryView();
        },

        postCreate: function() {
            var t = this;

            t.registerToggledNode(t.presetControlContainer);
            t.filterView.showList.push(t.presetControlContainer);
            t.summarizationView.showList.push(t.presetControlContainer);

            t.inherited(arguments);

            if (t.report.hasInteractivePresentation()) {
                t.own(aspect.after(t,"_setResultsAttr",function(results) {
                    t.summarizationEditor.set("modified",false);
                },true));
            }
        },

        initializeDisplay: function() {
            var t = this;
            if (t.report.hasInteractivePresentation()) {
                t.setView(t.gridView);
                if (t.report.crossTabDefaultView === "Summarization" && (t.report.summarizableColumns.toString().split(",").length > 1 || t.report.rowSummarizableColumns.toString().split(",").length > 1)) {
                    t.pushView(t.summarizationView);
                } else if (t.report.crossTabDefaultView === "Filter" && t.report.filterColumns !== "") {
                    t.pushView(t.filterView);
                } else {
                    t.defer("refresh");
                }
            } else {
                t.filterView.showList.push(t.optionIconNode);
                t.filterView.showList.push(t.summarizationIconNode);
                t.setView(t.filterView);
            }
        },

        updateFromPreset: function(preset) {
            var t = this;
            t.inherited(arguments);

            if (preset !== null) {
                var summaryPieces = preset.summarizeBy.split(",");

                t.summarizationEditor.summaryRowField.set("value",summaryPieces[0]);
                t.summarizationEditor.summaryColumnField.set("value",summaryPieces[1]);
                t.summarizationEditor.aggregatesField.set("value",(preset.aggregations && preset.aggregations.length > 0 ? preset.aggregations[0] : ""));
            }
            t.defer(function() {
                t.summarizationEditor.set("modified",false);
            });
        },

        createGrid: function() {
            var t = this;
            t.set("gridView",new CrossTabGridView({
                controller: t,
                title: t.report.displayName
            }));
        },

        _createSummaryView: function() {
            var t = this;

            t.summarizationEditor = new ObjectEditor({
                templateString: template.autoGenerateCrossTabSummarizationTemplate(t.report),
                report: t.report,
                controller: t,
                object: new Stateful({}),

                buildRendering: function() {
                    var t = this;
                    t.inherited("buildRendering",arguments);

                    if (t.report) {
                        var summarizableColumnStore = new Memory({
                            getLabel: function(item) {return item.name;}
                        });
                        arrayUtils.forEach(t.report.summarizableColumns.toString().split(","),function(column) {
                            var column = t.report.getColumn(column);
                            if (column) {
                                summarizableColumnStore.put({
                                    id: column.name,
                                    name: column.displayName
                                });
                            }
                        });
                        t.summaryColumnField = new Select({
                            store: summarizableColumnStore
                        },t.summaryColumnNode);

                        var summarizableColumnStore = new Memory({
                            getLabel: function(item) {return item.name;}
                        });
                        arrayUtils.forEach(t.report.rowSummarizableColumns.toString().split(","),function(column) {
                            var column = t.report.getColumn(column);
                            if (column) {
                                summarizableColumnStore.put({
                                    id: column.name,
                                    name: column.displayName
                                });
                            }
                        });
                        t.summaryRowField = new Select({
                            store: summarizableColumnStore
                        },t.summaryRowNode);
            
                        t.summaryRowField.name = "summaryRow";
                        t.own(on(t.summaryRowField,"change",function(value) {t._modifyValue(this.name);}));

                        t.summaryColumnField.name = "summaryColumn";
                        t.own(on(t.summaryColumnField,"change",function(value) {t._modifyValue(this.name);}));
					
						t.summarizeByField = t.summaryColumnField;

                        var aggregateStore = new Memory({
                            data: [
                                {id:"_COUNT",field:"_COUNT",name:"Count",column:"",aggregateFunction:"COUNT"},
                                {id:"_PERCENTCOUNT",field:"_PERCENTCOUNT",name:"% Count",column:"",aggregateFunction:"PERCENT COUNT"}
                            ],
                            getLabel: function(item) {return item.name;}
                        });
                        arrayUtils.forEach(t.report.columnDefinitions,function(definition) {
                            if ((definition.isDisplayed || definition.isFilterable) && definition.aggregateFunctions !== "") {
                                arrayUtils.forEach(definition.aggregateFunctions.toString().split(","),function(aggFunc) {
                                    aggregateStore.put({
                                        id: aggFunc+"||"+definition.name,
                                        field: aggFunc+definition.name,
                                        column: definition.name,
                                        aggregateFunction: aggFunc,
                                        name: t.controller.getAggregateName(aggFunc)+" "+definition.displayName
                                    });
                                });
                            }
                        });

                        t.aggregatesField = new Select({
                            store: aggregateStore
                        },t.aggregatesNode);
                        t.aggregatesField.name = "aggregates";
                        t.own(on(t.aggregatesField,"change",function(value) {t._modifyValue(this.name);}));
                        t.aggregatesField.startup();
                    }
                },

                postCreate: function() {
                    var t = this;
                    t.inherited("postCreate",arguments);

                    var checkSummarization = function(fieldName) {
                        if (t.summaryColumnField.get("value") === t.summaryRowField.get("value")) {
                            var field = t[fieldName+"Field"];

                            var newValue = "";
                            arrayUtils.some(field.store.data,function(item) {
                                if (item.id !== field.get("value")) {
                                    newValue = item.id;
                                    return true;
                                }
                            });

                            field.set("value",newValue);
                        }
                    };

                    t.registerChangeLogic("summaryColumn",lang.partial(checkSummarization,"summaryRow"));
                    t.registerChangeLogic("summaryRow",lang.partial(checkSummarization,"summaryColumn"));
                }
            });

            t.summarizationContainer.appendChild(t.summarizationEditor.domNode);
            t.summarizationEditor._postModify();

            t.summarizationView.isModified = function() {
                return t.summarizationEditor.get("modified");
            };
        },

        getAggregateName: function(aggregateFunction) {
            return SummarizableController.prototype.getAggregateName(aggregateFunction);
        },

        getContext: function(presentationType,report) {
            var t = this;
            var context = t.inherited(arguments);
            context.options.summarizeBy = t.summarizationEditor.summaryRowField.get("value")+","+t.summarizationEditor.summaryColumnField.get("value");
            context.aggregations = [t.summarizationEditor.aggregatesField.store.get(t.summarizationEditor.aggregatesField.get("value"))];


            if (t.viewStack.selectedChildWidget === t.gridView.detailGridView) {
                var detailGridView = t.gridView.detailGridView;
                context.options.rowValue = detailGridView.rowValue;
                context.options.columnValue = detailGridView.columnValue;
                context.sort = detailGridView.getSort();

                context.options.presetName = (typeof context.options.presetName !== "undefined" ? context.options.presetName : t.report.displayName)+" Details";
            }

            return context;
        },

        onApplySummarization: function() {
            var t = this;

            if (t.summarizationEditor.okToSave()) {
                t.set("search","");
                if (t.summarizationEditor.get("modified")) {
                    var currentPreset = t.presetManager.currentPreset;
                    currentPreset.summarizeBy = t.summarizationEditor.summaryRowField.get("value")+","+t.summarizationEditor.summaryColumnField.get("value");
                    currentPreset.aggregations = [t.summarizationEditor.aggregatesField.get("value")];
                    t.setCurrentPresetModified();

                    t.setModified("summarization");
                    t.summarizationEditor.set("modified",false);
                }

                if (t.getView() === t.summarizationView.id) {
                    t.popView();
                }
            }
        },

        onCancelSummarization: function() {
            var t = this;
            t.onCancel(t.summarizationView);
            t.summarizationEditor.set("modified",false);
            t.popView();
        }
	});
	
	return CrossTabReportController;
});